@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2014 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -29,12 +29,6 @@
 #include "url-scanner.h"
 #include "gmime-filter-html.h"
 
-#ifdef ENABLE_WARNINGS
-#define w(x) x
-#else
-#define w(x)
-#endif /* ENABLE_WARNINGS */
-
 #define d(x)
 
 
@@ -57,17 +51,11 @@ static struct {
 } patterns[] = {
 	{ CONVERT_WEB_URLS, { "file://",   "",        url_file_start,     url_file_end     } },
 	{ CONVERT_WEB_URLS, { "ftp://",    "",        url_web_start,      url_web_end      } },
-	{ CONVERT_WEB_URLS, { "sftp://",   "",        url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "http://",   "",        url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "https://",  "",        url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "news://",   "",        url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "nntp://",   "",        url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "telnet://", "",        url_web_start,      url_web_end      } },
-	{ CONVERT_WEB_URLS, { "webcal://", "",        url_web_start,      url_web_end      } },
-	{ CONVERT_WEB_URLS, { "mailto:",   "",        url_web_start,      url_web_end      } },
-	{ CONVERT_WEB_URLS, { "callto:",   "",        url_web_start,      url_web_end      } },
-	{ CONVERT_WEB_URLS, { "h323:",     "",        url_web_start,      url_web_end      } },
-	{ CONVERT_WEB_URLS, { "sip:",      "",        url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "www.",      "http://", url_web_start,      url_web_end      } },
 	{ CONVERT_WEB_URLS, { "ftp.",      "ftp://",  url_web_start,      url_web_end      } },
 	{ CONVERT_ADDRSPEC, { "@",         "mailto:", url_addrspec_start, url_addrspec_end } },
@@ -180,7 +168,7 @@ check_size (GMimeFilter *filter, char *outptr, char **outend, size_t len)
 }
 
 static int
-citation_depth (const char *in, const char *inend)
+citation_depth (const char *in)
 {
 	register const char *inptr = in;
 	int depth = 1;
@@ -192,11 +180,11 @@ citation_depth (const char *in, const char *inend)
 	if (!strncmp (inptr, "From", 4))
 		return 0;
 	
-	while (inptr < inend && *inptr != '\n') {
+	while (*inptr != '\n') {
 		if (*inptr == ' ')
 			inptr++;
 		
-		if (inptr >= inend || *inptr++ != '>')
+		if (*inptr++ != '>')
 			break;
 		
 		depth++;
@@ -266,7 +254,7 @@ writeln (GMimeFilter *filter, const char *in, const char *end, char *outptr, cha
 		u = html_utf8_getc (&inptr, inend);
 		switch (u) {
 		case 0xffff:
-			w(g_warning ("Invalid UTF-8 sequence encountered"));
+			g_warning ("Invalid UTF-8 sequence encountered");
 			return outptr;
 			break;
 		case '<':
@@ -333,7 +321,7 @@ html_convert (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 	
 	g_mime_filter_set_size (filter, inlen * 2 + 6, FALSE);
 	
-	start = inptr = in;
+	inptr = in;
 	inend = in + inlen;
 	outptr = filter->outbuf;
 	outend = filter->outbuf + filter->outsize;
@@ -343,23 +331,21 @@ html_convert (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 		html->pre_open = TRUE;
 	}
 	
-	do {
-		while (inptr < inend && *inptr != '\n')
-			inptr++;
-		
-		if (inptr == inend && !flush)
-			break;
-		
+	start = inptr;
+	while (inptr < inend && *inptr != '\n')
+		inptr++;
+	
+	while (inptr < inend) {
 		html->column = 0;
 		depth = 0;
 		
 		if (html->flags & GMIME_FILTER_HTML_MARK_CITATION) {
-			if ((depth = citation_depth (start, inend)) > 0) {
+			if ((depth = citation_depth (start)) > 0) {
 				char font[25];
 				
 				/* FIXME: we could easily support multiple colour depths here */
 				
-				g_snprintf (font, 25, "<font color=\"#%06x\">", (html->colour & 0xffffff));
+				g_snprintf (font, 25, "<font color=\"#%06x\">", html->colour);
 				
 				outptr = check_size (filter, outptr, &outend, 25);
 				outptr = g_stpcpy (outptr, font);
@@ -430,13 +416,18 @@ html_convert (GMimeFilter *filter, char *in, size_t inlen, size_t prespace,
 			outptr = g_stpcpy (outptr, "<br>");
 		}
 		
-		if (inptr < inend)
-			*outptr++ = '\n';
+		*outptr++ = '\n';
 		
 		start = ++inptr;
-	} while (inptr < inend);
+		while (inptr < inend && *inptr != '\n')
+			inptr++;
+	}
 	
 	if (flush) {
+		/* flush the rest of our input buffer */
+		if (start < inend)
+			outptr = writeln (filter, start, inend, outptr, &outend);
+		
 		if (html->pre_open) {
 			/* close the pre-tag */
 			outptr = check_size (filter, outptr, &outend, 10);
@@ -489,17 +480,17 @@ filter_reset (GMimeFilter *filter)
 GMimeFilter *
 g_mime_filter_html_new (guint32 flags, guint32 colour)
 {
-	GMimeFilterHTML *filter;
+	GMimeFilterHTML *new;
 	guint i;
 	
-	filter = g_object_newv (GMIME_TYPE_FILTER_HTML, 0, NULL);
-	filter->flags = flags;
-	filter->colour = colour;
+	new = g_object_newv (GMIME_TYPE_FILTER_HTML, 0, NULL);
+	new->flags = flags;
+	new->colour = colour;
 	
 	for (i = 0; i < NUM_URL_PATTERNS; i++) {
 		if (patterns[i].mask & flags)
-			url_scanner_add (filter->scanner, &patterns[i].pattern);
+			url_scanner_add (new->scanner, &patterns[i].pattern);
 	}
 	
-	return (GMimeFilter *) filter;
+	return (GMimeFilter *) new;
 }
