@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /*  GMime
- *  Copyright (C) 2000-2011 Jeffrey Stedfast
+ *  Copyright (C) 2000-2012 Jeffrey Stedfast
  *
  *  This library is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU Lesser General Public License
@@ -48,12 +48,14 @@ typedef unsigned int nfds_t;
 #endif
 
 #include "gmime-gpg-context.h"
+#ifdef ENABLE_CRYPTOGRAPHY
 #include "gmime-filter-charset.h"
 #include "gmime-stream-filter.h"
 #include "gmime-stream-pipe.h"
 #include "gmime-stream-mem.h"
 #include "gmime-stream-fs.h"
 #include "gmime-charset.h"
+#endif /* ENABLE_CRYPTOGRAPHY */
 #include "gmime-error.h"
 
 #ifdef ENABLE_DEBUG
@@ -166,8 +168,6 @@ g_mime_gpg_context_class_init (GMimeGpgContextClass *klass)
 static void
 g_mime_gpg_context_init (GMimeGpgContext *ctx, GMimeGpgContextClass *klass)
 {
-	GMimeCryptoContext *crypto = (GMimeCryptoContext *) ctx;
-	
 	ctx->auto_key_retrieve = FALSE;
 	ctx->always_trust = FALSE;
 	ctx->use_agent = FALSE;
@@ -268,6 +268,7 @@ gpg_get_key_exchange_protocol (GMimeCryptoContext *ctx)
 	return "application/pgp-keys";
 }
 
+#ifdef ENABLE_CRYPTOGRAPHY
 enum _GpgCtxMode {
 	GPG_CTX_MODE_SIGN,
 	GPG_CTX_MODE_VERIFY,
@@ -326,8 +327,9 @@ struct _GpgCtx {
 	unsigned int armor:1;
 	unsigned int need_passwd:1;
 	unsigned int bad_passwds:2;
+	unsigned int decrypt_okay:1;
 	
-	unsigned int padding:20;
+	unsigned int padding:19;
 };
 
 static struct _GpgCtx *
@@ -341,6 +343,7 @@ gpg_ctx_new (GMimeGpgContext *ctx)
 	gpg->mode = GPG_CTX_MODE_SIGN;
 	gpg->ctx = ctx;
 	gpg->userid_hint = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	gpg->decrypt_okay = FALSE;
 	gpg->complete = FALSE;
 	gpg->seen_eof1 = TRUE;
 	gpg->seen_eof2 = FALSE;
@@ -1286,7 +1289,8 @@ gpg_ctx_parse_status (struct _GpgCtx *gpg, GError **err)
 				/* second token is the cipher algorithm */
 				gpg->cipher = strtoul (status, &inend, 10);
 			} else if (!strncmp (status, "DECRYPTION_OKAY", 15)) {
-				/* nothing to do... but good to know gpg decrypted the data successfully */
+				/* decryption succeeded */
+				gpg->decrypt_okay = TRUE;
 			} else if (!strncmp (status, "DECRYPTION_FAILED", 17)) {
 				/* nothing to do... but we know gpg failed to decrypt :-( */
 			} else if (!strncmp (status, "END_DECRYPTION", 14)) {
@@ -1805,12 +1809,13 @@ gpg_ctx_op_wait (struct _GpgCtx *gpg)
 	else
 		return -1;
 }
-
+#endif /* ENABLE_CRYPTOGRAPHY */
 
 static int
 gpg_sign (GMimeCryptoContext *context, const char *userid, GMimeDigestAlgo digest,
 	  GMimeStream *istream, GMimeStream *ostream, GError **err)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
 	
@@ -1861,6 +1866,11 @@ gpg_sign (GMimeCryptoContext *context, const char *userid, GMimeDigestAlgo diges
 	gpg_ctx_free (gpg);
 	
 	return digest;
+#else
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("PGP support is not enabled in this build"));
+	
+	return -1;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 
@@ -1869,11 +1879,10 @@ gpg_verify (GMimeCryptoContext *context, GMimeDigestAlgo digest,
 	    GMimeStream *istream, GMimeStream *sigstream,
 	    GError **err)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	GMimeSignatureList *signatures;
-	const char *diagnostics;
 	struct _GpgCtx *gpg;
-	gboolean valid;
 	
 	gpg = gpg_ctx_new (ctx);
 	gpg_ctx_set_mode (gpg, GPG_CTX_MODE_VERIFY);
@@ -1918,6 +1927,11 @@ gpg_verify (GMimeCryptoContext *context, GMimeDigestAlgo digest,
 	gpg_ctx_free (gpg);
 	
 	return signatures;
+#else
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("PGP support is not enabled in this build"));
+	
+	return NULL;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 
@@ -1926,6 +1940,7 @@ gpg_encrypt (GMimeCryptoContext *context, gboolean sign, const char *userid,
 	     GMimeDigestAlgo digest, GPtrArray *recipients, GMimeStream *istream,
 	     GMimeStream *ostream, GError **err)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
 	guint i;
@@ -1983,6 +1998,11 @@ gpg_encrypt (GMimeCryptoContext *context, gboolean sign, const char *userid,
 	gpg_ctx_free (gpg);
 	
 	return 0;
+#else
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("PGP support is not enabled in this build"));
+	
+	return -1;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 
@@ -1990,6 +2010,7 @@ static GMimeDecryptResult *
 gpg_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 	     GMimeStream *ostream, GError **err)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	GMimeDecryptResult *result;
 	const char *diagnostics;
@@ -2020,7 +2041,7 @@ gpg_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 		}
 	}
 	
-	if (gpg_ctx_op_wait (gpg) != 0) {
+	if (gpg_ctx_op_wait (gpg) != 0 && !gpg->decrypt_okay) {
 		save = errno;
 		diagnostics = gpg_ctx_get_diagnostics (gpg);
 		errno = save;
@@ -2042,11 +2063,17 @@ gpg_decrypt (GMimeCryptoContext *context, GMimeStream *istream,
 	gpg_ctx_free (gpg);
 	
 	return result;
+#else
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("PGP support is not enabled in this build"));
+	
+	return NULL;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 static int
 gpg_import_keys (GMimeCryptoContext *context, GMimeStream *istream, GError **err)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
 	
@@ -2089,11 +2116,17 @@ gpg_import_keys (GMimeCryptoContext *context, GMimeStream *istream, GError **err
 	gpg_ctx_free (gpg);
 	
 	return 0;
+#else
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("PGP support is not enabled in this build"));
+	
+	return -1;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 static int
 gpg_export_keys (GMimeCryptoContext *context, GPtrArray *keys, GMimeStream *ostream, GError **err)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeGpgContext *ctx = (GMimeGpgContext *) context;
 	struct _GpgCtx *gpg;
 	guint i;
@@ -2142,6 +2175,11 @@ gpg_export_keys (GMimeCryptoContext *context, GPtrArray *keys, GMimeStream *ostr
 	gpg_ctx_free (gpg);
 	
 	return 0;
+#else
+	g_set_error (err, GMIME_ERROR, GMIME_ERROR_NOT_SUPPORTED, _("PGP support is not enabled in this build"));
+	
+	return -1;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 
@@ -2157,6 +2195,7 @@ gpg_export_keys (GMimeCryptoContext *context, GPtrArray *keys, GMimeStream *ostr
 GMimeCryptoContext *
 g_mime_gpg_context_new (GMimePasswordRequestFunc request_passwd, const char *path)
 {
+#ifdef ENABLE_CRYPTOGRAPHY
 	GMimeCryptoContext *crypto;
 	GMimeGpgContext *ctx;
 	
@@ -2169,6 +2208,9 @@ g_mime_gpg_context_new (GMimePasswordRequestFunc request_passwd, const char *pat
 	crypto->request_passwd = request_passwd;
 	
 	return crypto;
+#else
+	return NULL;
+#endif /* ENABLE_CRYPTOGRAPHY */
 }
 
 
